@@ -1,13 +1,20 @@
-from datetime import datetime
+import os
+import urllib.parse
+import urllib.request
 from typing import Optional, List
+from datetime import datetime
 
-from IPython.core.display import display
-from mlflow.store.tracking import SEARCH_MAX_RESULTS_DEFAULT
-from flown.api.wrapper_for_notebook import notebook_api
-from flown.utils import merge_html, s3util
-from mlflow.tracking import MlflowClient
-from mlflow.entities import ViewType
 import markdown
+from ipywidgets import Image
+
+from mlflow.entities import ViewType
+from mlflow.server import ARTIFACT_ROOT_ENV_VAR
+from mlflow.store.tracking import SEARCH_MAX_RESULTS_DEFAULT
+from mlflow.tracking import MlflowClient
+
+from flown import utils
+from flown.api.wrapper_for_notebook import notebook_api
+from flown.utils import merge_html
 
 _client = MlflowClient()
 _md = markdown.Markdown()
@@ -21,7 +28,7 @@ def list_experiments(view_type=ViewType.ACTIVE_ONLY, restricted: bool = False, d
              experiment_id=exp.experiment_id,
              lifecycle_stage=exp.lifecycle_stage,
              artifact_location=exp.artifact_location,
-             artifact_console_url=s3util.s3uri_to_console_url(exp.artifact_location, is_obj=False),
+             artifact_console_url=utils.to_browsable_url(exp.artifact_location, is_obj=False),
              tags=exp.tags
              )
         for exp in experiments
@@ -61,7 +68,7 @@ def _disp_experiment(experiment_id: str,
     exp_info_data = dict(experiment_id=exp_info.experiment_id,
                          name=exp_info.name,
                          artifact_location=exp_info.artifact_location,
-                         artifact_console_url=s3util.s3uri_to_console_url(exp_info.artifact_location, is_obj=False),
+                         artifact_console_url=utils.to_browsable_url(exp_info.artifact_location, is_obj=False),
                          tags=exp_info.tags,
                          note=exp_info.tags.get('mlflow.note.content', ''),
                          note_html=_md.convert(exp_info.tags.get('mlflow.note.content', '')),
@@ -86,7 +93,7 @@ def _disp_experiment(experiment_id: str,
                     start_time=datetime.fromtimestamp(run.start_time / 1e3),
                     end_time=datetime.fromtimestamp(run.end_time / 1e3),
                     artifact_uri=run.artifact_uri,
-                    artifact_console_url=s3util.s3uri_to_console_url(run.artifact_uri, is_obj=False),
+                    artifact_console_url=utils.to_browsable_url(run.artifact_uri, is_obj=False),
                     lifecycle_stage=run.lifecycle_stage,
                     detail=detail,
                     children=[]
@@ -117,7 +124,8 @@ def _disp_experiment(experiment_id: str,
         for parent in current_list:
             parent['generation_index'] = current_index
             total_list.append(parent)
-            set_generation_index(total_list, current_index+1, parent['children'])
+            set_generation_index(total_list, current_index + 1, parent['children'])
+
     indexed_list = []
     set_generation_index(indexed_list, 0, root_runs)
 
@@ -132,18 +140,22 @@ def _disp_experiment(experiment_id: str,
 
 @notebook_api
 def disp_run(run_id: str,
+             artifact_path: str = None,
              restricted: bool = False,
              display_id: str = None
              ):
-    return _disp_run(run_id,
+    return _disp_run(run_id=run_id,
+                     artifact_path=artifact_path,
                      restricted=restricted,
                      display_id=display_id)
 
 
 def _disp_run(run_id: str,
+              artifact_path: str = None,
               restricted: bool = False,
               display_id: str = None
               ):
+    # 選択されたrunの情報を取得
     run = _client.get_run(run_id=run_id)
     run_info = run.info
     run_data = dict(run_uuid=run_info.run_uuid,
@@ -153,31 +165,43 @@ def _disp_run(run_id: str,
                     experiment_id=run_info.experiment_id,
                     user_id=run_info.user_id,
                     status=run_info.status,
-                    start_time=run_info.start_time,
-                    end_time=run_info.end_time,
+                    start_time=datetime.fromtimestamp(run_info.start_time / 1e3),
+                    end_time=datetime.fromtimestamp(run_info.end_time / 1e3),
                     artifact_uri=run_info.artifact_uri,
-                    artifact_console_url=s3util.s3uri_to_console_url(run_info.artifact_uri, is_obj=False),
+                    artifact_console_url=utils.to_browsable_url(run_info.artifact_uri, is_obj=False),
                     lifecycle_stage=run_info.lifecycle_stage,
                     tags=run.data.tags,
                     note=run.data.tags.get('mlflow.note.content', ''),
                     note_html=_md.convert(run.data.tags.get('mlflow.note.content', '')),
                     params=run.data.params,
                     metrics=run.data.metrics,
+                    artifacts=[dict(path=a.path, file_size=a.file_size, is_selected=(a.path == artifact_path))
+                               for a in _client.list_artifacts(run_id=run_id)],
                     )
 
+    # 選択されたアーティファクトのビューワHTMLを生成
+    if artifact_path:
+        selected_artifact_html = merge_html('artifact_viewer.html',
+                                            dict(artifact_path=artifact_path))
+    else:
+        selected_artifact_html = ''
+
+    # runにひもづく実験の情報を取得
     exp_info = _client.get_experiment(experiment_id=run_info.experiment_id)
     exp_info_data = dict(experiment_id=exp_info.experiment_id,
                          name=exp_info.name,
                          artifact_location=exp_info.artifact_location,
-                         artifact_console_url=s3util.s3uri_to_console_url(exp_info.artifact_location, is_obj=False),
+                         artifact_console_url=utils.to_browsable_url(exp_info.artifact_location, is_obj=False),
                          tags=exp_info.tags,
                          note=exp_info.tags.get('mlflow.note.content', ''),
                          note_html=_md.convert(exp_info.tags.get('mlflow.note.content', '')),
                          )
+    Image()
 
     return merge_html(template_name='run_detail.html',
                       params={'exp_info': exp_info_data,
                               'run_data': run_data,
+                              'selected_artifact_html': selected_artifact_html
                               },
                       display_id=display_id)
 
@@ -222,3 +246,47 @@ def update_run_note(run_id: str, note_str: str,
                     key='mlflow.note.content',
                     value=note_str)
     return _disp_run(run_id, restricted=restricted, display_id=display_id)
+
+
+# uriをOS任せ（ブラウザなど）で開く。 画面更新なしなので @notebook_api で修飾しない。
+def startfile(uri: str, restricted: bool = False, display_id: str = None):
+    if uri.startswith('http:') or uri.startswith('https:') or uri.startswith('ftp:'):
+        os.startfile(uri)
+    else:
+        path = urllib.parse.urlparse(uri).path if uri.startswith("file:") else uri
+        path = urllib.request.url2pathname(path)
+        os.startfile(path)
+
+
+def set_default_artifact_uri_root(default_artifact_root: str):
+    os.environ[ARTIFACT_ROOT_ENV_VAR] = default_artifact_root
+
+
+@notebook_api
+def text_editor(text_type: str, entity_id: str,
+                    restricted: bool = False,
+                    display_id: str = None
+                    ):
+    if text_type == 'run_name':
+        run = _client.get_run(run_id=entity_id)
+        text_value = run.data.tags.get('mlflow.runName', '')
+    else:
+        raise Exception(f'unknown text_type : {text_type}')
+
+    return merge_html(template_name='text_editor.html',
+                      params={'text_type': text_type,
+                              'entity_id': entity_id,
+                              'text_value': text_value,
+                              },
+                      display_id=display_id)
+
+
+@notebook_api
+def update_run_name(run_id: str, name_value: str,
+                    restricted: bool = False,
+                    display_id: str = None
+                    ):
+    _client.set_tag(run_id=run_id,
+                    key='mlflow.runName',
+                    value=name_value)
+    return _disp_run(run_id, restricted=restricted, display_id=display_id)
